@@ -148,8 +148,9 @@ SEXP OGR_write(SEXP inp)
             }
             uninstallErrorHandlerAndTriggerError();
             installErrorHandler();
-            if (!strcmp(CHAR(STRING_ELT(VECTOR_ELT(inp, 3), 0)),
-                "ESRI Shapefile")) hSRS.morphToESRI();
+            if (LOGICAL_POINTER(VECTOR_ELT(inp, 11))[0]) {
+                    hSRS.morphToESRI();
+            }
             uninstallErrorHandlerAndTriggerError();
             installErrorHandler();
             poLayer = poDS->CreateLayer( CHAR(STRING_ELT(VECTOR_ELT(inp, 2),
